import argparse

import requests_mock

METGET_DMY_ENDPOINT = "https://metget.server.dmy"
METGET_DMY_APIKEY = "1234567890"
METGET_API_VERSION = 2

METGET_CREDIT_RESPONSE_JSON = {
    "statusCode": 200,
    "body": {"credit_limit": 0.0, "credits_used": 13707.9168, "credit_balance": 0.0},
}

METGET_CREDIT_RESPONSE_TEXT = (
    "+--------------+--------------+----------------+\n"
    "| Credit Limit | Credits Used | Credit Balance |\n"
    "+--------------+--------------+----------------+\n"
    "|  Unlimited   |  13707.9168  |   Unlimited    |\n"
    "+--------------+--------------+----------------+\n"
)

METGET_FORMATTED_CREDIT_RESPONSE_JSON = {
    "credit_limit": "Unlimited",
    "credits_used": 13707.9168,
    "credit_balance": "Unlimited",
}


def test_credits(capfd) -> None:
    """
    Tests the credits data
    Args:
        capfd: pytest fixture to capture stdout and stderr

    Returns:
        None
    """
    import json

    from metget.metget_credits import metget_credits

    args = argparse.Namespace()
    args.endpoint = METGET_DMY_ENDPOINT
    args.apikey = METGET_DMY_APIKEY
    args.api_version = METGET_API_VERSION
    args.format = "json"

    url = METGET_DMY_ENDPOINT + "/credits"
    with requests_mock.Mocker() as m:
        m.get(
            url,
            headers={"x-api-key": METGET_DMY_APIKEY},
            json=METGET_CREDIT_RESPONSE_JSON,
        )
        metget_credits(args)
        out, err = capfd.readouterr()
        out = out.replace("'", '"')
        assert json.loads(out) == METGET_FORMATTED_CREDIT_RESPONSE_JSON

        args.format = "pretty"
        metget_credits(args)
        out, err = capfd.readouterr()
        assert out == METGET_CREDIT_RESPONSE_TEXT



import argparse
from urllib.parse import urlencode

import requests_mock

from metget.metget_status import MetGetStatus

from .status_json import (
    COAMPS_CTCX_STATUS_JSON,
    GEFS_STATUS_JSON,
    GEFS_STATUS_JSON_C00,
    GFS_STATUS_JSON,
    HAFS_STATUS_JSON,
    HWRF_STATUS_BRET_JSON,
    HWRF_STATUS_JSON,
    NHC_STATUS_JSON,
)
from .status_text import (
    COAMPS_CTCX_STATUS_TEXT,
    GEFS_STATUS_TEXT,
    GEFS_STATUS_TEXT_C00,
    GFS_STATUS_TEXT,
    HAFS_STATUS_TEXT,
    HWRF_STATUS_BRET_TEXT,
    HWRF_STATUS_TEXT,
    NHC_STATUS_TEXT,
)

METGET_DMY_ENDPOINT = "https://metget.server.dmy"
METGET_DMY_APIKEY = "1234567890"
METGET_API_VERSION = 2


def test_get_status_gfs(capfd) -> None:
    """
    Tests the status command for the GFS model
    Args:
        capfd: pytest fixture to capture stdout and stderr

    Returns:
        None
    """
    import json

    args = argparse.Namespace()
    args.model = "gfs"
    args.format = "pretty"
    args.start = None
    args.end = None
    args.complete = False
    args.endpoint = METGET_DMY_ENDPOINT
    args.apikey = METGET_DMY_APIKEY
    args.api_version = METGET_API_VERSION

    s = MetGetStatus(args)
    with requests_mock.Mocker() as m:
        m.get(METGET_DMY_ENDPOINT + "/status?model=gfs", json=GFS_STATUS_JSON)
        s.get_status()
    out, err = capfd.readouterr()
    assert out == GFS_STATUS_TEXT

    args.format = "json"

    s = MetGetStatus(args)
    with requests_mock.Mocker() as m:
        m.get(METGET_DMY_ENDPOINT + "/status?model=gfs", json=GFS_STATUS_JSON)
        s.get_status()
        out, err = capfd.readouterr()
        out_dict = json.loads(out)
        assert out_dict == GFS_STATUS_JSON["body"]


def test_status_hwrf(capfd) -> None:
    """
    Tests the status command for the HWRF model

    Args:
        capfd: pytest fixture to capture stdout and stderr

    Returns:
        None
    """
    import json
    from datetime import datetime

    args = argparse.Namespace()
    args.model = "hwrf"
    args.format = "pretty"
    args.start = datetime(2023, 6, 1)
    args.storm = None
    args.complete = False
    args.end = datetime(2023, 7, 11)
    args.endpoint = METGET_DMY_ENDPOINT
    args.apikey = METGET_DMY_APIKEY
    args.api_version = METGET_API_VERSION

    s = MetGetStatus(args)
    url = (
        METGET_DMY_ENDPOINT
        + "/status?"
        + urlencode({"model": "hwrf", "start": "2023-06-01", "end": "2023-07-11"})
    )
    with requests_mock.Mocker() as m:
        m.get(url, json=HWRF_STATUS_JSON)
        s.get_status()
    out, err = capfd.readouterr()
    assert out == HWRF_STATUS_TEXT

    args.format = "json"

    s = MetGetStatus(args)
    with requests_mock.Mocker() as m:
        m.get(url, json=HWRF_STATUS_JSON)
        s.get_status()
        out, err = capfd.readouterr()
        out_dict = json.loads(out)
        assert out_dict == HWRF_STATUS_JSON["body"]

    args.storm = "bret03l"
    s2 = MetGetStatus(args)
    url = (
        METGET_DMY_ENDPOINT
        + "/status?"
        + urlencode(
            {
                "model": "hwrf",
                "start": "2023-06-01",
                "end": "2023-07-11",
                "storm": "bret03l",
            }
        )
    )
    with requests_mock.Mocker() as m:
        m.get(url, json=HWRF_STATUS_BRET_JSON)
        s2.get_status()
        out, err = capfd.readouterr()
        out_dict = json.loads(out)
        assert out_dict == HWRF_STATUS_BRET_JSON["body"]

    args.format = "pretty"
    s3 = MetGetStatus(args)
    with requests_mock.Mocker() as m:
        m.get(url, json=HWRF_STATUS_BRET_JSON)
        s3.get_status()
        out, err = capfd.readouterr()
        assert out == HWRF_STATUS_BRET_TEXT

    args.year = "2023"
    s4 = MetGetStatus(args)
    with requests_mock.Mocker() as m:
        m.get(url, json=HWRF_STATUS_BRET_JSON)
        s4.get_status()
        out, err = capfd.readouterr()
        assert out == HWRF_STATUS_BRET_TEXT


def test_status_nhc(capfd) -> None:
    """
    Tests the status command for the NHC data
    Args:
        capfd: pytest fixture to capture stdout and stderr

    Returns:
        None
    """
    import json
    from datetime import datetime

    args = argparse.Namespace()
    args.model = "nhc"
    args.start = datetime(2023, 6, 1)
    args.end = None
    args.storm = None
    args.basin = None
    args.format = "json"
    args.endpoint = METGET_DMY_ENDPOINT
    args.apikey = METGET_DMY_APIKEY
    args.api_version = METGET_API_VERSION

    with requests_mock.Mocker() as m:
        m.get(
            METGET_DMY_ENDPOINT
            + "/status?"
            + urlencode({"model": "nhc", "start": "2023-06-01"}),
            json=NHC_STATUS_JSON,
        )
        s = MetGetStatus(args)
        s.get_status()
        out, err = capfd.readouterr()
        assert json.loads(out) == NHC_STATUS_JSON["body"]

    args.format = "pretty"
    with requests_mock.Mocker() as m:
        m.get(
            METGET_DMY_ENDPOINT
            + "/status?"
            + urlencode({"model": "nhc", "start": "2023-06-01"}),
            json=NHC_STATUS_JSON,
        )
        s = MetGetStatus(args)
        s.get_status()
        out, err = capfd.readouterr()
        assert out == NHC_STATUS_TEXT


def test_status_gefs(capfd) -> None:
    """
    Tests the status command for the GEFS model
    Args:
        capfd: pytest fixture to capture stdout and stderr

    Returns:
        None
    """
    import json
    from datetime import datetime

    args = argparse.Namespace()
    args.model = "gefs"
    args.start = datetime(2023, 6, 1)
    args.end = datetime(2023, 6, 8)
    args.ensemble_member = "c00"
    args.format = "json"
    args.storm = None
    args.complete = False
    args.endpoint = METGET_DMY_ENDPOINT
    args.apikey = METGET_DMY_APIKEY
    args.api_version = METGET_API_VERSION

    with requests_mock.Mocker() as m:
        m.get(
            METGET_DMY_ENDPOINT
            + "/status?"
            + urlencode(
                {
                    "model": "gefs",
                    "start": "2023-06-01",
                    "end": "2023-06-08",
                    "member": "c00",
                }
            ),
            json=GEFS_STATUS_JSON_C00,
        )
        s = MetGetStatus(args)
        s.get_status()
        out, err = capfd.readouterr()
        assert json.loads(out) == GEFS_STATUS_JSON_C00["body"]["c00"]

    args.format = "pretty"
    with requests_mock.Mocker() as m:
        m.get(
            METGET_DMY_ENDPOINT
            + "/status?"
            + urlencode(
                {
                    "model": "gefs",
                    "start": "2023-06-01",
                    "end": "2023-06-08",
                    "member": "c00",
                }
            ),
            json=GEFS_STATUS_JSON_C00,
        )
        s = MetGetStatus(args)
        s.get_status()
        out, err = capfd.readouterr()
        assert out == GEFS_STATUS_TEXT_C00

    args.ensemble_member = None
    args.format = "json"
    with requests_mock.Mocker() as m:
        m.get(
            METGET_DMY_ENDPOINT
            + "/status?"
            + urlencode({"model": "gefs", "start": "2023-06-01", "end": "2023-06-08"}),
            json=GEFS_STATUS_JSON,
        )
        s = MetGetStatus(args)
        s.get_status()
        out, err = capfd.readouterr()
        assert json.loads(out) == GEFS_STATUS_JSON["body"]

    args.format = "pretty"
    with requests_mock.Mocker() as m:
        m.get(
            METGET_DMY_ENDPOINT
            + "/status?"
            + urlencode({"model": "gefs", "start": "2023-06-01", "end": "2023-06-08"}),
            json=GEFS_STATUS_JSON,
        )
        s = MetGetStatus(args)
        s.get_status()
        out, err = capfd.readouterr()
        assert out == GEFS_STATUS_TEXT


def test_status_ctcx(capfd) -> None:
    """
    Tests the status command for the COAMPS-CTCX model
    Args:
        capfd:

    Returns:
        None
    """
    import json
    from datetime import datetime

    args = argparse.Namespace()
    args.model = "ctcx"
    args.start = datetime(2022, 9, 26)
    args.end = datetime(2022, 9, 30)
    args.format = "json"
    args.storm = None
    args.endpoint = METGET_DMY_ENDPOINT
    args.apikey = METGET_DMY_APIKEY
    args.api_version = METGET_API_VERSION

    with requests_mock.Mocker() as m:
        m.get(
            METGET_DMY_ENDPOINT
            + "/status?"
            + urlencode(
                {
                    "model": "ctcx",
                    "start": "2022-09-26",
                    "end": "2022-09-30",
                }
            ),
            json=COAMPS_CTCX_STATUS_JSON,
        )
        s = MetGetStatus(args)
        s.get_status()
        out, err = capfd.readouterr()
        assert json.loads(out) == COAMPS_CTCX_STATUS_JSON["body"]

    args.format = "pretty"
    with requests_mock.Mocker() as m:
        m.get(
            METGET_DMY_ENDPOINT
            + "/status?"
            + urlencode(
                {
                    "model": "ctcx",
                    "start": "2022-09-26",
                    "end": "2022-09-30",
                }
            ),
            json=COAMPS_CTCX_STATUS_JSON,
        )
        s = MetGetStatus(args)
        s.get_status()
        out, err = capfd.readouterr()
        assert out == COAMPS_CTCX_STATUS_TEXT


def test_get_status_hafs(capfd) -> None:
    """
    Tests the status command for the HAFS-A model

    Args:
        capfd: pytest fixture to capture stdout and stderr

    Returns:
        None
    """
    import json
    from datetime import datetime

    args = argparse.Namespace()
    args.model = "hafsa"
    args.format = "pretty"
    args.start = datetime(2024, 1, 11)
    args.storm = None
    args.end = datetime(2024, 1, 18)
    args.endpoint = METGET_DMY_ENDPOINT
    args.apikey = METGET_DMY_APIKEY
    args.api_version = METGET_API_VERSION

    s = MetGetStatus(args)
    url = (
        METGET_DMY_ENDPOINT
        + "/status?"
        + urlencode({"model": "hafsa", "start": "2024-01-11", "end": "2024-01-18"})
    )
    with requests_mock.Mocker() as m:
        m.get(url, json=HAFS_STATUS_JSON)
        s.get_status()
    out, err = capfd.readouterr()
    assert out == HAFS_STATUS_TEXT

    args.format = "json"

    s = MetGetStatus(args)
    with requests_mock.Mocker() as m:
        m.get(url, json=HAFS_STATUS_JSON)
        s.get_status()
        out, err = capfd.readouterr()
        out_dict = json.loads(out)
        assert out_dict == HAFS_STATUS_JSON["body"]



import argparse
from datetime import datetime

import pytest
import requests_mock

from metget.metget_build import MetGetBuildRest
from metget.metget_environment import get_metget_environment_variables

from .build_json import (
    METGET_BUILD_COAMPS_JSON,
    METGET_BUILD_CTCX_JSON,
    METGET_BUILD_GEFS_JSON,
    METGET_BUILD_GFS_JSON,
    METGET_BUILD_HWRF_JSON,
    METGET_BUILD_NHC_JSON,
    METGET_BUILD_NHC_JSON_THIS_YEAR,
    METGET_BUILD_POST_RETURN,
    METGET_BUILD_RETURN_COMPLETE,
    METGET_BUILD_RETURN_QUEUED,
    METGET_BUILD_RETURN_RESTORE,
    METGET_BUILD_RETURN_RUNNING,
)

METGET_DMY_ENDPOINT = "https://metget.server.dmy"
METGET_DMY_APIKEY = "1234567890"
METGET_API_VERSION = 2


def test_build_gfs(capfd) -> None:
    """
    Tests the build request for a single domain of gfs data
    Args:
        capfd: pytest fixture to capture stdout and stderr

    Returns:
        None
    """

    args = argparse.Namespace()
    args.analysis = False
    args.multiple_forecasts = True
    args.data_type = "wind_pressure"
    args.backfill = False
    args.domain = [["gfs", 0.25, -100, 10, -80, 30]]
    args.start = datetime(2023, 6, 1)
    args.end = datetime(2023, 6, 2)
    args.initialization_skip = 0
    args.timestep = 3600
    args.format = None
    args.output = "test_build_gfs"
    args.dryrun = False
    args.strict = False
    args.epsg = 4326
    args.check_interval = 1
    args.max_wait = 3600
    args.output_directory = None

    args.endpoint = METGET_DMY_ENDPOINT
    args.apikey = METGET_DMY_APIKEY
    args.api_version = METGET_API_VERSION

    request_dict = MetGetBuildRest.generate_request_json(
        analysis=args.analysis,
        multiple_forecasts=args.multiple_forecasts,
        start_date=args.start,
        end_date=args.end,
        format=args.format,
        timestep=args.timestep,
        data_type=args.data_type,
        backfill=args.backfill,
        filename=args.output,
        dry_run=args.dryrun,
        strict=args.strict,
        domains=MetGetBuildRest.parse_command_line_domains(
            args.domain, args.initialization_skip
        ),
    )

    request_dict["creator"] = "pytest"
    assert request_dict == METGET_BUILD_GFS_JSON

    environment = get_metget_environment_variables(args)
    client = MetGetBuildRest(
        environment["endpoint"], environment["apikey"], environment["api_version"]
    )

    with requests_mock.Mocker() as m:
        m.post(METGET_DMY_ENDPOINT + "/build", json=METGET_BUILD_POST_RETURN)
        data_id, status_code = client.make_metget_request(request_dict)

    assert data_id == "5f9b5b3c-5b7a-4c5e-8b0a-1b5b3c5d7e8f"
    assert status_code == 200

    metget_request_mocker(
        data_id,
        client,
        ["test_build_gfs_00.pre", "test_build_gfs_00.wnd"],
        "filelist_gfs.json",
        args,
        capfd,
    )


def test_build_hwrf_multidomain(capfd) -> None:
    """
    Tests the build request for a multi domain hwrf+gfs data
    Args:
        capfd: pytest fixture to capture stdout and stderr

    Returns:
        None
    """
    args = argparse.Namespace()
    args.analysis = False
    args.multiple_forecasts = True
    args.data_type = "wind_pressure"
    args.backfill = False
    args.domain = [
        ["gfs", 0.25, -100, 10, -70, 30],
        ["hwrf-bret03l", 0.1, -90, 15, -80, 25],
    ]
    args.start = datetime(2023, 6, 19)
    args.end = datetime(2023, 6, 24)
    args.initialization_skip = 0
    args.timestep = 3600
    args.format = None
    args.output = "test_build_hwrf"
    args.dryrun = False
    args.strict = False
    args.epsg = 4326
    args.check_interval = 1
    args.max_wait = 3600
    args.output_directory = None

    args.endpoint = METGET_DMY_ENDPOINT
    args.apikey = METGET_DMY_APIKEY
    args.api_version = METGET_API_VERSION

    request_dict = MetGetBuildRest.generate_request_json(
        analysis=args.analysis,
        multiple_forecasts=args.multiple_forecasts,
        start_date=args.start,
        end_date=args.end,
        format=args.format,
        timestep=args.timestep,
        data_type=args.data_type,
        backfill=args.backfill,
        filename=args.output,
        dry_run=args.dryrun,
        strict=args.strict,
        domains=MetGetBuildRest.parse_command_line_domains(
            args.domain, args.initialization_skip
        ),
    )

    request_dict["creator"] = "pytest"
    assert request_dict == METGET_BUILD_HWRF_JSON

    environment = get_metget_environment_variables(args)
    client = MetGetBuildRest(
        environment["endpoint"], environment["apikey"], environment["api_version"]
    )

    with requests_mock.Mocker() as m:
        m.post(METGET_DMY_ENDPOINT + "/build", json=METGET_BUILD_POST_RETURN)
        data_id, status_code = client.make_metget_request(request_dict)

    assert data_id == "5f9b5b3c-5b7a-4c5e-8b0a-1b5b3c5d7e8f"
    assert status_code == 200

    metget_request_mocker(
        data_id,
        client,
        [
            "metget_test_bret03l_00.pre",
            "metget_test_bret03l_00.wnd",
            "metget_test_bret03l_01.pre",
            "metget_test_bret03l_01.wnd",
        ],
        "filelist_hwrf.json",
        args,
        capfd,
    )


def test_build_nhc_raw(capfd) -> None:
    """
    Tests the build request for a raw nhc data
    Args:
        capfd: pytest fixture to capture stdout and stderr

    Returns:
        None
    """
    import json
    import os

    from metget.metget_build import metget_build

    args = argparse.Namespace()
    args.analysis = False
    args.multiple_forecasts = True
    args.variable = "wind_pressure"
    args.backfill = False
    args.domain = [["nhc-2023-al-09-015", 0.1, -90, 15, -80, 25]]
    args.start = datetime(2023, 6, 19)
    args.end = datetime(2023, 6, 24)
    args.initialization_skip = 0
    args.timestep = 3600
    args.format = "raw"
    args.output = "test_build_nhc"
    args.dryrun = False
    args.strict = False
    args.epsg = 4326
    args.check_interval = 1
    args.max_wait = 3600
    args.request = None
    args.compression = False
    args.save_json_request = True
    args.output_directory = None

    args.endpoint = METGET_DMY_ENDPOINT
    args.apikey = METGET_DMY_APIKEY
    args.api_version = METGET_API_VERSION

    request_dict = MetGetBuildRest.generate_request_json(
        analysis=args.analysis,
        multiple_forecasts=args.multiple_forecasts,
        start_date=args.start,
        end_date=args.end,
        format=args.format,
        timestep=args.timestep,
        data_type=args.variable,
        backfill=args.backfill,
        filename=args.output,
        dry_run=args.dryrun,
        strict=args.strict,
        domains=MetGetBuildRest.parse_command_line_domains(
            args.domain, args.initialization_skip
        ),
    )

    request_dict["creator"] = "pytest"

    assert request_dict == METGET_BUILD_NHC_JSON

    environment = get_metget_environment_variables(args)
    client = MetGetBuildRest(
        environment["endpoint"], environment["apikey"], environment["api_version"]
    )

    with requests_mock.Mocker() as m:
        m.post(METGET_DMY_ENDPOINT + "/build", json=METGET_BUILD_POST_RETURN)
        data_id, status_code = client.make_metget_request(request_dict)

    assert data_id == "5f9b5b3c-5b7a-4c5e-8b0a-1b5b3c5d7e8f"
    assert status_code == 200

    output_files = [
        "nhc_merge_2022_al_09_015.trk",
        "nhc_btk_2022_al_09.btk",
        "nhc_fcst_2022_al_09_015.fcst",
    ]

    metget_request_mocker(
        data_id,
        client,
        output_files,
        "filelist_nhc.json",
        args,
        capfd,
    )

    path = os.path.join(
        os.path.dirname(os.path.abspath(__file__)),
        "test_files",
        "filelist_nhc.json",
    )

    # ...Do the same thing with the raw function
    with requests_mock.Mocker() as m:
        response_list = [
            {"json": METGET_BUILD_RETURN_QUEUED, "status_code": 200},
            {"json": METGET_BUILD_RETURN_QUEUED, "status_code": 200},
            {"json": METGET_BUILD_RETURN_RUNNING, "status_code": 200},
            {"json": METGET_BUILD_RETURN_COMPLETE, "status_code": 200},
        ]
        m.post(METGET_DMY_ENDPOINT + "/build", json=METGET_BUILD_POST_RETURN)
        m.get(
            METGET_DMY_ENDPOINT + f"/check?request-id={data_id:s}",
            response_list,
        )
        data_id, status_code = client.make_metget_request(request_dict)
        with open(path, "rb") as filelist_file:
            filelist_data = json.load(filelist_file)
            m.get(
                f"https://s3.amazonaws.com/metget/{data_id:s}/filelist.json",
                json=filelist_data,
            )

            for file in output_files:
                m.get(
                    f"https://s3.amazonaws.com/metget/{data_id:s}/{file:s}",
                    text="This is only a test",
                )

            metget_build(args)

            for file in output_files:
                assert os.path.exists(file)
                os.remove(file)
            os.remove("filelist.json")
            os.remove("request.json")


def test_build_nhc_raw_thisyear(capfd) -> None:
    """
    Tests the build request for a raw nhc data for the current year
    Args:
        capfd: pytest fixture to capture stdout and stderr

    Returns:
        None
    """
    import json
    import os

    from metget.metget_build import metget_build

    args = argparse.Namespace()
    args.analysis = False
    args.multiple_forecasts = True
    args.variable = "wind_pressure"
    args.backfill = False
    args.domain = [["nhc-al-09-015", 0.1, -90, 15, -80, 25]]
    args.start = datetime(2023, 6, 19)
    args.end = datetime(2023, 6, 24)
    args.initialization_skip = 0
    args.timestep = 3600
    args.format = "raw"
    args.output = "test_build_nhc"
    args.dryrun = False
    args.strict = False
    args.epsg = 4326
    args.check_interval = 1
    args.max_wait = 3600
    args.request = None
    args.compression = False
    args.save_json_request = True
    args.output_directory = None

    args.endpoint = METGET_DMY_ENDPOINT
    args.apikey = METGET_DMY_APIKEY
    args.api_version = METGET_API_VERSION

    request_dict = MetGetBuildRest.generate_request_json(
        analysis=args.analysis,
        multiple_forecasts=args.multiple_forecasts,
        start_date=args.start,
        end_date=args.end,
        format=args.format,
        timestep=args.timestep,
        data_type=args.variable,
        backfill=args.backfill,
        filename=args.output,
        dry_run=args.dryrun,
        strict=args.strict,
        domains=MetGetBuildRest.parse_command_line_domains(
            args.domain, args.initialization_skip
        ),
    )

    request_dict["creator"] = "pytest"

    assert request_dict == METGET_BUILD_NHC_JSON_THIS_YEAR

    environment = get_metget_environment_variables(args)
    client = MetGetBuildRest(
        environment["endpoint"], environment["apikey"], environment["api_version"]
    )

    with requests_mock.Mocker() as m:
        m.post(METGET_DMY_ENDPOINT + "/build", json=METGET_BUILD_POST_RETURN)
        data_id, status_code = client.make_metget_request(request_dict)

    assert data_id == "5f9b5b3c-5b7a-4c5e-8b0a-1b5b3c5d7e8f"
    assert status_code == 200

    output_files = [
        "nhc_merge_2022_al_09_015.trk",
        "nhc_btk_2022_al_09.btk",
        "nhc_fcst_2022_al_09_015.fcst",
    ]

    metget_request_mocker(
        data_id,
        client,
        output_files,
        "filelist_nhc.json",
        args,
        capfd,
    )

    path = os.path.join(
        os.path.dirname(os.path.abspath(__file__)),
        "test_files",
        "filelist_nhc.json",
    )

    # ...Do the same thing with the raw function
    with requests_mock.Mocker() as m:
        response_list = [
            {"json": METGET_BUILD_RETURN_QUEUED, "status_code": 200},
            {"json": METGET_BUILD_RETURN_QUEUED, "status_code": 200},
            {"json": METGET_BUILD_RETURN_RUNNING, "status_code": 200},
            {"json": METGET_BUILD_RETURN_COMPLETE, "status_code": 200},
        ]
        m.post(METGET_DMY_ENDPOINT + "/build", json=METGET_BUILD_POST_RETURN)
        m.get(
            METGET_DMY_ENDPOINT + f"/check?request-id={data_id:s}",
            response_list,
        )
        data_id, status_code = client.make_metget_request(request_dict)
        with open(path, "rb") as filelist_file:
            filelist_data = json.load(filelist_file)
            m.get(
                f"https://s3.amazonaws.com/metget/{data_id:s}/filelist.json",
                json=filelist_data,
            )

            for file in output_files:
                m.get(
                    f"https://s3.amazonaws.com/metget/{data_id:s}/{file:s}",
                    text="This is only a test",
                )

            metget_build(args)

            for file in output_files:
                assert os.path.exists(file)
                os.remove(file)
            os.remove("filelist.json")
            os.remove("request.json")


def test_build_nhc_raw_error(capfd) -> None:
    """
    Tests the build request for a nhc data that is not correctly
    formatted. xfail.

    Args:
        capfd: pytest fixture to capture stdout and stderr

    Returns:
        None
    """
    args = argparse.Namespace()
    args.analysis = False
    args.multiple_forecasts = True
    args.variable = "wind_pressure"
    args.backfill = False
    args.domain = [["nhc-al", 0.1, -90, 15, -80, 25]]
    args.start = datetime(2023, 6, 19)
    args.end = datetime(2023, 6, 24)
    args.initialization_skip = 0
    args.timestep = 3600
    args.format = "raw"
    args.output = "test_build_nhc"
    args.dryrun = False
    args.strict = False
    args.epsg = 4326
    args.check_interval = 1
    args.max_wait = 3600
    args.request = None
    args.compression = False
    args.save_json_request = True
    args.output_directory = None

    args.endpoint = METGET_DMY_ENDPOINT
    args.apikey = METGET_DMY_APIKEY
    args.api_version = METGET_API_VERSION

    with pytest.raises(RuntimeError):
        MetGetBuildRest.generate_request_json(
            analysis=args.analysis,
            multiple_forecasts=args.multiple_forecasts,
            start_date=args.start,
            end_date=args.end,
            format=args.format,
            timestep=args.timestep,
            data_type=args.variable,
            backfill=args.backfill,
            filename=args.output,
            dry_run=args.dryrun,
            strict=args.strict,
            domains=MetGetBuildRest.parse_command_line_domains(
                args.domain, args.initialization_skip
            ),
        )


def test_build_hwrf_multidomain_error(capfd) -> None:
    """
    Tests the build request for a multi domain hwrf+gfs data
    which is not correctly formatted. xfail.

    Args:
        capfd: pytest fixture to capture stdout and stderr

    Returns:
        None
    """
    args = argparse.Namespace()
    args.analysis = False
    args.multiple_forecasts = True
    args.data_type = "wind_pressure"
    args.backfill = False
    args.domain = [
        ["gfs", 0.25, -100, 10, -70, 30],
        ["hwrf", 0.1, -90, 15, -80, 25],
    ]
    args.start = datetime(2023, 6, 19)
    args.end = datetime(2023, 6, 24)
    args.initialization_skip = 0
    args.timestep = 3600
    args.format = None
    args.output = "test_build_hwrf"
    args.dryrun = False
    args.strict = False
    args.epsg = 4326
    args.check_interval = 1
    args.max_wait = 3600
    args.output_directory = None

    args.endpoint = METGET_DMY_ENDPOINT
    args.apikey = METGET_DMY_APIKEY
    args.api_version = METGET_API_VERSION

    with pytest.raises(RuntimeError):
        MetGetBuildRest.generate_request_json(
            analysis=args.analysis,
            multiple_forecasts=args.multiple_forecasts,
            start_date=args.start,
            end_date=args.end,
            format=args.format,
            timestep=args.timestep,
            data_type=args.data_type,
            backfill=args.backfill,
            filename=args.output,
            dry_run=args.dryrun,
            strict=args.strict,
            domains=MetGetBuildRest.parse_command_line_domains(
                args.domain, args.initialization_skip
            ),
        )


def test_build_gefs(capfd) -> None:
    """
    Tests the build request for a single domain of gefs data

    Args:
        capfd: pytest fixture to capture stdout and stderr

    Returns:
        None
    """
    args = argparse.Namespace()
    args.analysis = False
    args.multiple_forecasts = True
    args.data_type = "wind_pressure"
    args.backfill = False
    args.domain = [["gefs-c00", 0.25, -100, 10, -80, 30]]
    args.start = datetime(2023, 6, 1)
    args.end = datetime(2023, 6, 2)
    args.initialization_skip = 0
    args.timestep = 3600
    args.format = None
    args.output = "test_build_gefs"
    args.dryrun = False
    args.strict = False
    args.epsg = 4326
    args.check_interval = 1
    args.max_wait = 3600
    args.output_directory = None

    args.endpoint = METGET_DMY_ENDPOINT
    args.apikey = METGET_DMY_APIKEY
    args.api_version = METGET_API_VERSION

    request_dict = MetGetBuildRest.generate_request_json(
        analysis=args.analysis,
        multiple_forecasts=args.multiple_forecasts,
        start_date=args.start,
        end_date=args.end,
        format=args.format,
        timestep=args.timestep,
        data_type=args.data_type,
        backfill=args.backfill,
        filename=args.output,
        dry_run=args.dryrun,
        strict=args.strict,
        domains=MetGetBuildRest.parse_command_line_domains(
            args.domain, args.initialization_skip
        ),
    )

    request_dict["creator"] = "pytest"
    assert request_dict == METGET_BUILD_GEFS_JSON

    environment = get_metget_environment_variables(args)
    client = MetGetBuildRest(
        environment["endpoint"], environment["apikey"], environment["api_version"]
    )

    with requests_mock.Mocker() as m:
        m.post(METGET_DMY_ENDPOINT + "/build", json=METGET_BUILD_POST_RETURN)
        data_id, status_code = client.make_metget_request(request_dict)

    assert data_id == "5f9b5b3c-5b7a-4c5e-8b0a-1b5b3c5d7e8f"
    assert status_code == 200

    metget_request_mocker(
        data_id,
        client,
        ["test_build_gefs_00.pre", "test_build_gefs_00.wnd"],
        "filelist_gefs.json",
        args,
        capfd,
    )


def test_build_coamps(capfd) -> None:
    """
    Tests the build request for a single domain of coamps data

    Args:
        capfd: pytest fixture to capture stdout and stderr

    Returns:
        None
    """
    args = argparse.Namespace()
    args.analysis = False
    args.multiple_forecasts = True
    args.data_type = "wind_pressure"
    args.backfill = False
    args.domain = [["coamps-09L", 0.25, -100, 10, -80, 30]]
    args.start = datetime(2023, 6, 1)
    args.end = datetime(2023, 6, 2)
    args.initialization_skip = 0
    args.timestep = 3600
    args.format = None
    args.output = "test_build_coamps"
    args.tau = 0
    args.dryrun = False
    args.strict = False
    args.epsg = 4326
    args.check_interval = 1
    args.max_wait = 3600
    args.output_directory = None

    args.endpoint = METGET_DMY_ENDPOINT
    args.apikey = METGET_DMY_APIKEY
    args.api_version = METGET_API_VERSION

    request_dict = MetGetBuildRest.generate_request_json(
        analysis=args.analysis,
        multiple_forecasts=args.multiple_forecasts,
        start_date=args.start,
        end_date=args.end,
        format=args.format,
        timestep=args.timestep,
        data_type=args.data_type,
        backfill=args.backfill,
        filename=args.output,
        dry_run=args.dryrun,
        strict=args.strict,
        domains=MetGetBuildRest.parse_command_line_domains(
            args.domain, args.initialization_skip
        ),
    )

    request_dict["creator"] = "pytest"
    assert request_dict == METGET_BUILD_COAMPS_JSON

    environment = get_metget_environment_variables(args)
    client = MetGetBuildRest(
        environment["endpoint"], environment["apikey"], environment["api_version"]
    )

    with requests_mock.Mocker() as m:
        m.post(METGET_DMY_ENDPOINT + "/build", json=METGET_BUILD_POST_RETURN)
        data_id, status_code = client.make_metget_request(request_dict)

    assert data_id == "5f9b5b3c-5b7a-4c5e-8b0a-1b5b3c5d7e8f"
    assert status_code == 200

    metget_request_mocker(
        data_id,
        client,
        ["test_build_gefs_00.pre", "test_build_gefs_00.wnd"],
        "filelist_gefs.json",
        args,
        capfd,
    )


def test_build_ctcx(capfd) -> None:
    """
    Tests the build request for a single domain of ctcx data

    Args:
        capfd: pytest fixture to capture stdout and stderr

    Returns:
        None
    """
    args = argparse.Namespace()
    args.analysis = False
    args.multiple_forecasts = True
    args.data_type = "wind_pressure"
    args.backfill = False
    args.domain = [["ctcx-09L-01", 0.25, -100, 10, -80, 30]]
    args.start = datetime(2023, 6, 1)
    args.end = datetime(2023, 6, 2)
    args.initialization_skip = 4
    args.timestep = 3600
    args.format = None
    args.output = "test_build_ctcx"
    args.dryrun = False
    args.strict = False
    args.epsg = 4326
    args.check_interval = 1
    args.max_wait = 3600
    args.output_directory = None

    args.endpoint = METGET_DMY_ENDPOINT
    args.apikey = METGET_DMY_APIKEY
    args.api_version = METGET_API_VERSION

    request_dict = MetGetBuildRest.generate_request_json(
        analysis=args.analysis,
        multiple_forecasts=args.multiple_forecasts,
        start_date=args.start,
        end_date=args.end,
        format=args.format,
        timestep=args.timestep,
        data_type=args.data_type,
        backfill=args.backfill,
        filename=args.output,
        dry_run=args.dryrun,
        strict=args.strict,
        domains=MetGetBuildRest.parse_command_line_domains(
            args.domain, args.initialization_skip
        ),
    )

    request_dict["creator"] = "pytest"
    assert request_dict == METGET_BUILD_CTCX_JSON

    environment = get_metget_environment_variables(args)
    client = MetGetBuildRest(
        environment["endpoint"], environment["apikey"], environment["api_version"]
    )

    with requests_mock.Mocker() as m:
        m.post(METGET_DMY_ENDPOINT + "/build", json=METGET_BUILD_POST_RETURN)
        data_id, status_code = client.make_metget_request(request_dict)

    assert data_id == "5f9b5b3c-5b7a-4c5e-8b0a-1b5b3c5d7e8f"
    assert status_code == 200

    metget_request_mocker(
        data_id,
        client,
        ["test_build_gefs_00.pre", "test_build_gefs_00.wnd"],
        "filelist_gefs.json",
        args,
        capfd,
    )


def test_build_gfs_error(capfd) -> None:
    """
    Tests the build request for a single domain of gfs data
    which is not correctly formatted. Should raise.

    Args:
        capfd: pytest fixture to capture stdout and stderr

    Returns:
        None
    """

    args = argparse.Namespace()
    args.analysis = False
    args.multiple_forecasts = True
    args.data_type = "wind_pressure"
    args.backfill = False
    args.domain = [["gfs-01", 0.25, -100, 10, -80, 30]]
    args.start = datetime(2023, 6, 1)
    args.end = datetime(2023, 6, 2)
    args.initialization_skip = 0
    args.timestep = 3600
    args.format = None
    args.output = "test_build_gfs"
    args.dryrun = False
    args.strict = False
    args.epsg = 4326
    args.check_interval = 1
    args.max_wait = 3600
    args.output_directory = None

    args.endpoint = METGET_DMY_ENDPOINT
    args.apikey = METGET_DMY_APIKEY
    args.api_version = METGET_API_VERSION

    with pytest.raises(RuntimeError):
        MetGetBuildRest.generate_request_json(
            analysis=args.analysis,
            multiple_forecasts=args.multiple_forecasts,
            start_date=args.start,
            end_date=args.end,
            format=args.format,
            timestep=args.timestep,
            data_type=args.data_type,
            backfill=args.backfill,
            filename=args.output,
            dry_run=args.dryrun,
            strict=args.strict,
            domains=MetGetBuildRest.parse_command_line_domains(
                args.domain, args.initialization_skip
            ),
        )


def metget_request_mocker(
    data_id: str,
    client: MetGetBuildRest,
    output_files: list,
    filelist_file: str,
    args: argparse.Namespace,
    capfd,
) -> None:
    import json
    import os

    response_list = [
        {"json": METGET_BUILD_RETURN_QUEUED, "status_code": 200},
        {"json": METGET_BUILD_RETURN_QUEUED, "status_code": 200},
        {"json": METGET_BUILD_RETURN_RESTORE, "status_code": 200},
        {"json": METGET_BUILD_RETURN_RUNNING, "status_code": 200},
        {"json": METGET_BUILD_RETURN_COMPLETE, "status_code": 200},
    ]

    with requests_mock.Mocker() as m:
        m.get(
            METGET_DMY_ENDPOINT + f"/check?request-id={data_id:s}",
            response_list,
        )
        path = os.path.join(
            os.path.dirname(os.path.abspath(__file__)),
            "test_files",
            filelist_file,
        )
        with open(path, "rb") as filelist_file:
            filelist_data = json.load(filelist_file)
            m.get(
                f"https://s3.amazonaws.com/metget/{data_id:s}/filelist.json",
                json=filelist_data,
            )

            for file in output_files:
                m.get(
                    f"https://s3.amazonaws.com/metget/{data_id:s}/{file:s}",
                    text="This is only a test",
                )

            client.download_metget_data(
                data_id, args.check_interval, args.max_wait, args.output_directory
            )
            out, err = capfd.readouterr()

        # ...Clean up
        for file in output_files:
            os.remove(file)
        os.remove("filelist.json")



###################################################################################################
# MIT License
#
# Copyright (c) 2023 The Water Institute
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author: Zach Cobell
# Contact: zcobell@thewaterinstitute.org
# Organization: The Water Institute
#
###################################################################################################
import argparse


class MetGetStatus:
    def __init__(self, args: argparse.Namespace):
        """
        This method is used to initialize the MetGet status command

        Args:
            args: The arguments passed to the command line

        Returns:
            None
        """
        from .metget_environment import get_metget_environment_variables

        self.__args = args
        self.__environment = get_metget_environment_variables(args)
        self.__model_class = None

        if self.__environment["api_version"] != 2:
            msg = "Only API version 2 is supported."
            raise RuntimeError(msg)

    def get_status(self) -> None:
        """
        This method is used to get the status of the metget data
        """
        from .metget_data import MODEL_TYPES

        model = self.__args.model
        self.__model_class = MODEL_TYPES[model]

        if self.__model_class == "synoptic":
            self.__status_synoptic(model)
        elif self.__model_class == "synoptic-storm":
            self.__status_synoptic_storm(model)
        elif self.__model_class == "ensemble":
            self.__status_ensemble(model)
        elif self.__model_class == "ensemble-storm":
            self.__status_ensemble_storm(model)
        # elif self.__model_class == "hindcast":
        #     self.__status_hindcast(model)
        elif self.__model_class == "track":
            self.__status_track(model)
        else:
            msg = "Unknown model type."
            raise RuntimeError(msg)

    def __add_url_start_end_parameters(self, url):
        """
        This method is used to add the start and end parameters to the url

        Args:
            url: The url to add the start and end parameters to

        Returns:
            The url with the start and end parameters added
        """
        from datetime import datetime, timezone

        if self.__args.start:
            url += "&start={:s}".format(self.__args.start.strftime("%Y-%m-%d"))
            if not self.__args.end:
                url += "&end={:s}".format(
                    datetime.now(timezone.utc).strftime("%Y-%m-%d")
                )
            else:
                url += "&end={:s}".format(self.__args.end.strftime("%Y-%m-%d"))
        return url

    # TODO: ERA5 Hindcast
    # def __status_hindcast(self, model: str) -> None:
    #     import json
    #     from datetime import datetime, timedelta
    #
    #     import prettytable
    #     import requests
    #
    #     url = "{:s}/status?model={:s}".format(self.__environment["endpoint"], model)
    #     url = self.__add_url_start_end_parameters(url)
    #
    #     response = requests.get(
    #         url, headers={"x-api-key": self.__environment["apikey"]}
    #     )
    #     data = response.json()["body"]
    #
    #     if self.__args.format == "json":
    #         print(json.dumps(data))
    #     else:
    #         print(f"Status for model: {model:s} (class: {self.__model_class:s})")
    #         table = prettytable.PrettyTable(["Forecast Cycle", "End Time", "Status"])
    #         for cycle in data["cycles"]:
    #             cycle_date = datetime.fromisoformat(cycle["cycle"])
    #             end_time = cycle_date + timedelta(hours=cycle["duration"])
    #             if cycle["cycle"] in data["cycles_complete"]:
    #                 status = "Complete"
    #             else:
    #                 status = "Incomplete ({:d})".format(cycle["duration"])
    #             table.add_row(
    #                 [
    #                     cycle["cycle"],
    #                     end_time.strftime("%Y-%m-%d %H:%M:%S"),
    #                     status,
    #                 ]
    #             )
    #
    #         print(table)

    def __status_track(self, model: str) -> None:
        """
        This method is used to get the status of the track data (NHC)

        Args:
            model: The model to get the status for
        """
        import json

        import prettytable
        import requests

        url = "{:s}/status?model={:s}".format(self.__environment["endpoint"], model)
        url = self.__add_url_start_end_parameters(url)
        if self.__args.storm:
            url += f"&storm={self.__args.storm:s}"
        if self.__args.basin:
            url += f"&basin={self.__args.basin:s}"

        response = requests.get(
            url, headers={"x-api-key": self.__environment["apikey"]}
        )
        data = response.json()["body"]

        if self.__args.format == "json":
            print(json.dumps(data))
        elif self.__args.format == "pretty":
            storm_tracks = []
            for year in data["best_track"]:
                for basin in data["best_track"][year]:
                    for storm in data["best_track"][year][basin]:
                        start_time = data["best_track"][year][basin][storm][
                            "best_track_start"
                        ]
                        end_time = data["best_track"][year][basin][storm][
                            "best_track_end"
                        ]
                        duration = data["best_track"][year][basin][storm]["duration"]

                        if year in data["forecast"]:
                            if basin in data["forecast"][year]:
                                if storm in data["forecast"][year][basin]:
                                    advisories = list(
                                        data["forecast"][year][basin][storm].keys()
                                    )
                                else:
                                    advisories = []
                            else:
                                advisories = []
                        else:
                            advisories = []
                        storm_tracks.append(
                            {
                                "year": year,
                                "basin": basin,
                                "storm": storm,
                                "start_time": start_time,
                                "end_time": end_time,
                                "duration": duration,
                                "advisories": advisories,
                            }
                        )
            table = prettytable.PrettyTable(
                [
                    "Year",
                    "Basin",
                    "Storm",
                    "Best Track Start",
                    "Best Track End",
                    "Duration (hrs)",
                    "Forecast Advisories",
                ]
            )
            for storm_track in storm_tracks:
                advisories = storm_track["advisories"]
                if len(advisories) > 6:
                    advisories = advisories[:3] + ["..."] + advisories[-3:]
                elif len(advisories) == 0:
                    advisories = ["None"]

                # ...Convert the list to a string
                advisories = " ".join(advisories)

                table.add_row(
                    [
                        storm_track["year"],
                        storm_track["basin"],
                        storm_track["storm"],
                        storm_track["start_time"],
                        storm_track["end_time"],
                        storm_track["duration"],
                        advisories,
                    ]
                )
            table.reversesort = True
            print(table.get_string(sortby="Best Track End"))

        else:
            msg = "Unknown format"
            raise RuntimeError(msg)

    def __status_ensemble_storm(self, model: str) -> None:
        """
        This method is used to get the status of the ensemble storm data

        Args:
            model: The model to get the status for

        Returns:
            None
        """
        import json

        import prettytable
        import requests

        url = "{:s}/status?model={:s}".format(self.__environment["endpoint"], model)
        url = self.__add_url_start_end_parameters(url)
        if self.__args.storm:
            url += f"&storm={self.__args.storm:s}"
            if self.__args.ensemble_member:
                url += f"&member={self.__args.ensemble_member:s}"

        response = requests.get(
            url, headers={"x-api-key": self.__environment["apikey"]}
        )
        data = response.json()["body"]

        if self.__args.format == "json":
            print(json.dumps(data))
        elif self.__args.format == "pretty":
            if self.__args.storm and self.__args.ensemble_member:
                model_name = "{:s}-{:s}-{:s}".format(
                    model, self.__args.storm, self.__args.ensemble_member
                )
                self.__print_status_generic(
                    model_name,
                    data[self.__args.storm][self.__args.ensemble_member],
                    self.__args.complete,
                )
            else:
                table = prettytable.PrettyTable(
                    [
                        "Storm",
                        "First Forecast Cycle",
                        "Last Forecast Cycle",
                        "Earliest Forecast Time",
                        "Latest Forecast Time",
                        "Cycle Count",
                        "Ensemble Members",
                    ]
                )
                table.align["Ensemble Members"] = "l"

                for year in data:
                    for storm in data[year]:
                        ensemble_members = []

                        break_counter = 0
                        ensemble_members_str = ""
                        for ensemble_member in data[year][storm]:
                            ensemble_members.append(ensemble_member)
                            if break_counter == 0:
                                ensemble_members_str += f"{ensemble_member:s}"
                            else:
                                ensemble_members_str += f", {ensemble_member:s}"
                            break_counter += 1
                            if break_counter == 5:
                                ensemble_members_str += "\n"
                                break_counter = 0
                        table.add_row(
                            [
                                storm,
                                data[year][storm][ensemble_members[0]][
                                    "first_available_cycle"
                                ],
                                data[year][storm][ensemble_members[0]][
                                    "latest_available_cycle"
                                ],
                                data[year][storm][ensemble_members[0]][
                                    "min_forecast_date"
                                ],
                                data[year][storm][ensemble_members[0]][
                                    "max_forecast_date"
                                ],
                                len(data[year][storm][ensemble_members[0]]["cycles"]),
                                ensemble_members_str,
                            ]
                        )
                print(table)

    def __status_ensemble(self, model: str) -> None:
        """
        This method is used to get the status of the ensemble data

        Args:
            model: The model to get the status for

        Returns:
            None
        """
        import requests

        url = "{:s}/status?model={:s}".format(self.__environment["endpoint"], model)
        url = self.__add_url_start_end_parameters(url)
        if self.__args.ensemble_member:
            url += f"&member={self.__args.ensemble_member:s}"
        response = requests.get(
            url, headers={"x-api-key": self.__environment["apikey"]}
        )

        if self.__args.ensemble_member:
            model_name = f"{model:s}-{self.__args.ensemble_member:s}"
            self.__print_status_generic(
                model_name,
                response.json()["body"][self.__args.ensemble_member],
                self.__args.complete,
            )
        else:
            self.__print_status_multi("ensemble", model, response.json()["body"])

    def __status_synoptic(self, model: str) -> None:
        """
        This method is used to get the status of the synoptic data

        Args:
            model: The model to get the status for

        Returns:
            None
        """
        import requests

        url = "{:s}/status?model={:s}".format(self.__environment["endpoint"], model)
        url = self.__add_url_start_end_parameters(url)

        # ...Get the json from the endpoint
        response = requests.get(
            url, headers={"x-api-key": self.__environment["apikey"]}
        )

        self.__print_status_generic(
            model.upper(), response.json()["body"], self.__args.complete
        )

    def __status_synoptic_storm(self, model: str) -> None:
        """
        This method is used to get the status of the synoptic data for storms

        Args:
            model: The model to get the status for

        Returns:
            None
        """
        import json

        import requests

        url = "{:s}/status?model={:s}".format(self.__environment["endpoint"], model)
        url = self.__add_url_start_end_parameters(url)

        if self.__args.storm:
            url += f"&storm={self.__args.storm:s}"

        # ...Get the json from the endpoint
        response = requests.get(
            url, headers={"x-api-key": self.__environment["apikey"]}
        )

        # ...Print
        if response.json()["body"] == {}:
            if self.__args.format == "json":
                print(json.dumps(response.json()["body"]))
            else:
                print("No data found.")
        else:
            self.__print_status_multi("storm", model, response.json()["body"])

    def __print_status_multi(self, data_type: str, model: str, data: dict) -> None:
        """
        This method is used to print the status of the storm data

        Args:
            model: The model to get the status for
            data: The data to print the status for

        Returns:
            None
        """
        import json

        import prettytable

        if self.__args.format == "json":
            print(json.dumps(data))
        elif self.__args.format == "pretty":
            if data == {}:
                print("No data found.")
                return

            if self.__args.storm:
                model_name = f"{model.upper():s}-{self.__args.storm:s}"
                if len(data.keys()) > 1:
                    if "year" not in self.__args:
                        print("[ERROR]: Must provide a year")
                        exit(1)
                    else:
                        year = self.__args.year
                else:
                    year = next(iter(data.keys()))
                self.__print_status_generic(
                    model_name, data[year][self.__args.storm], self.__args.complete
                )
            else:
                if data_type == "ensemble":
                    table = prettytable.PrettyTable(
                        [
                            "Ensemble Member",
                            "First Forecast Cycle",
                            "Last Forecast Cycle",
                            "Earliest Forecast Time",
                            "Latest Forecast Time",
                            "Cycle Count",
                        ]
                    )
                    table.align["Ensemble Member"] = "r"
                elif data_type == "storm":
                    table = prettytable.PrettyTable(
                        [
                            "Storm",
                            "First Forecast Cycle",
                            "Last Forecast Cycle",
                            "Earliest Forecast Time",
                            "Latest Forecast Time",
                            "Cycle Count",
                        ]
                    )
                    table.align["Storm"] = "r"
                else:
                    msg = f"Unknown data type: {data_type:s}"
                    raise ValueError(msg)

                if data_type == "ensemble":
                    for it in data:
                        table.add_row(
                            [
                                it,
                                data[it]["first_available_cycle"],
                                data[it]["latest_available_cycle"],
                                data[it]["min_forecast_date"],
                                data[it]["max_forecast_date"],
                                len(data[it]["cycles"]),
                            ]
                        )
                    table.align["Cycle Count"] = "r"
                    print(f"Status for {model.upper():s} Model Ensemble Members")
                    print(table.get_string(sortby="Ensemble Member"))
                else:
                    for year in data:
                        for it in data[year]:
                            table.add_row(
                                [
                                    it,
                                    data[year][it]["first_available_cycle"],
                                    data[year][it]["latest_available_cycle"],
                                    data[year][it]["min_forecast_date"],
                                    data[year][it]["max_forecast_date"],
                                    len(data[year][it]["cycles"]),
                                ]
                            )
                    table.align["Cycle Count"] = "r"
                    table.reversesort = True
                    print(
                        "Status for {:s} Model Storms (class: {:s})".format(
                            model.upper(), self.__model_class
                        )
                    )
                    print(table.get_string(sortby="First Forecast Cycle"))

    def __print_status_generic(
        self, model: str, data: dict, only_complete: bool
    ) -> None:
        """
        This method is used to get the status of a generic format

        Args:
            model: The model to get the status for
            data: The data to get the status for

        Returns:
            None
        """
        import json
        from datetime import datetime, timedelta

        import prettytable

        if self.__args.format == "json":
            print(json.dumps(data))
        elif self.__args.format == "pretty":
            complete_cycles = data["cycles_complete"]

            print(f"Status for model: {model:s} (class: {self.__model_class:s})")
            table = prettytable.PrettyTable(["Forecast Cycle", "End Time", "Status"])

            for cycle in data["cycles"]:
                if cycle["cycle"] in complete_cycles:
                    status = "complete"
                else:
                    status = "incomplete ({:d})".format(cycle["duration"])

                if status == "complete" or not only_complete:
                    table.add_row(
                        [
                            cycle["cycle"],
                            datetime.strptime(cycle["cycle"], "%Y-%m-%d %H:%M:%S")
                            + timedelta(hours=cycle["duration"]),
                            status,
                        ]
                    )

            print(table)


def metget_status(args: argparse.Namespace) -> None:
    """
    This method is used to get the status of the metget data

    Args:
        args: The arguments passed to the command line

    Returns:
        None
    """
    s = MetGetStatus(args)
    return s.get_status()



###################################################################################################
# MIT License
#
# Copyright (c) 2023 The Water Institute
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author: Zach Cobell
# Contact: zcobell@thewaterinstitute.org
# Organization: The Water Institute
#
###################################################################################################
import argparse


class MetGetTrack:
    """
    This class is used to generate a MetGet track data from the api
    """

    def __init__(self, args: argparse.Namespace):
        from .metget_environment import get_metget_environment_variables

        self.__args = args
        self.__environment = get_metget_environment_variables(args)

    def get_track(self):
        """
        This method is used to get the track data from the api
        """
        import json
        from datetime import datetime

        import requests

        url = self.__environment["endpoint"] + "/stormtrack"

        if not self.__args.storm:
            msg = "Storm must be specified for track data"
            raise ValueError(msg)

        if not self.__args.type:
            msg = "Type must be specified for track data"
            raise ValueError(msg)

        # ...Default basin is Atlantic
        if not self.__args.basin:
            self.__args.basin = "al"

        # ...Default year is the current year
        if not self.__args.year:
            self.__args.year = datetime.now().year

        storm_id = f"{int(self.__args.storm):02d}"

        if self.__args.type == "besttrack":
            params = {
                "type": "best",
                "storm": storm_id,
                "basin": self.__args.basin,
                "year": self.__args.year,
            }
        elif self.__args.type == "forecast":
            if not self.__args.advisory:
                msg = "Advisory must be specified for forecast track data"
                raise ValueError(msg)

            advisory_id = f"{int(self.__args.advisory):03d}"
            params = {
                "type": "forecast",
                "storm": storm_id,
                "basin": self.__args.basin,
                "advisory": advisory_id,
                "year": self.__args.year,
            }
        else:
            msg = "Type must be besttrack or forecast"
            raise ValueError(msg)

        # Get the track data with the requests library
        response = requests.get(url, params=params)

        # Check the response status code
        if response.status_code == 200:
            print(json.dumps(response.json()["body"]["geojson"]))
        else:
            print(f"Error: {response.status_code}")
            print(json.dumps(response.json()))


def metget_track(args: argparse.Namespace) -> None:
    """
    This method is used to get the track data from the api

    Args:
        args: The command line arguments

    Returns:
        None
    """
    track = MetGetTrack(args)
    track.get_track()



###################################################################################################
# MIT License
#
# Copyright (c) 2023 The Water Institute
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author: Zach Cobell
# Contact: zcobell@thewaterinstitute.org
# Organization: The Water Institute
#
###################################################################################################
from typing import Optional


class SpinnerLogger:
    """
    Class to handle the spinner animation and logging
    """

    def __init__(self):
        """
        Constructor
        """
        from sys import stdout

        try:
            from halo import Halo

            self.__is_tty = stdout.isatty()
        except ImportError:
            if stdout.isatty():
                print(
                    "[WARNING]: Halo package not found. Disabling animation.",
                    flush=True,
                )
            self.__is_tty = False

        self.__current_text = SpinnerLogger.standard_log(0, "initializing")

        if self.__is_tty:
            self.__spinner = Halo(
                text=self.__current_text,
                spinner="dots",
                placement="left",
            )
            self.__spinner.color = self.__spinner.text_color

    @staticmethod
    def __time_str() -> str:
        """
        Returns the current time in UTC

        Returns:
            str: Current time in UTC
        """
        from datetime import datetime, timezone

        return datetime.now(timezone.utc).strftime("%Y-%m-%d %H:%M:%S UTC")

    def start(self, text: Optional[str] = None) -> None:
        """
        Starts the spinner animation

        Args:
            text (str, optional): Text to display. Defaults to None.

        Returns:
            None
        """
        if text is not None:
            self.__current_text = text
        if self.__is_tty:
            self.__spinner.start(self.__current_text)
        else:
            if text is not None:
                SpinnerLogger.__standard_print(self.__current_text)

    def set_text(self, text: str) -> None:
        """
        Sets the text to display in the spinner

        Args:
            text (str): Text to display

        Returns:
            None
        """
        self.__current_text = text
        if self.__is_tty:
            self.__spinner.text = self.__current_text
        else:
            if text is not None:
                SpinnerLogger.__standard_print(self.__current_text)

    def succeed(self, text: Optional[str] = None) -> None:
        """
        Stops the spinner animation and displays a success message

        Args:
            text (str, optional): Text to display. Defaults to None.

        Returns:
            None
        """
        if text is not None:
            self.__current_text = text

        if self.__is_tty:
            self.__spinner.succeed(self.__current_text)
        else:
            if text is not None:
                SpinnerLogger.__standard_print(self.__current_text)

    def info(self, text: Optional[str] = None) -> None:
        """
        Displays an info message in the spinner

        Args:
            text (str, optional): Text to display. Defaults to None.

        Returns:
            None
        """
        if text is not None:
            self.__current_text = text

        if self.__is_tty:
            self.__spinner.info(self.__current_text)
        else:
            if text is not None:
                SpinnerLogger.__standard_print(self.__current_text)

    def fail(self, text: Optional[str] = None) -> None:
        """
        Stops the spinner animation and displays a failure message

        Args:
            text (str, optional): Text to display. Defaults to None.

        Returns:
            None
        """
        if text is not None:
            self.__current_text = text

        if self.__is_tty:
            self.__spinner.fail(self.__current_text)
        else:
            if text is not None:
                SpinnerLogger.__standard_print(self.__current_text)

    @staticmethod
    def __standard_print(text: str) -> None:
        """
        Prints the given text to stdout

        Args:
            text (str): Text to print

        Returns:
            None
        """
        print(text, flush=True)

    @staticmethod
    def standard_log(count: int, status: str) -> str:
        """
        Returns a standard log message

        Args:
            count (int): Number of requests
            status (str): Status of the request

        Returns:
            str: Standard log message
        """
        return "[{:s}]: Checking request status...(n={:d}): {:s}".format(
            SpinnerLogger.__time_str(), count, status
        )



###################################################################################################
# MIT License
#
# Copyright (c) 2023 The Water Institute
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author: Zach Cobell
# Contact: zcobell@thewaterinstitute.org
# Organization: The Water Institute
#
###################################################################################################

# Available metget models
AVAILABLE_MODELS = {
    "gfs": "gfs-ncep",
    "gefs": "gefs-ncep",
    "nam": "nam-ncep",
    "hwrf": "hwrf",
    "hrrr": "hrrr-conus",
    "hrrr-alaska": "hrrr-alaska",
    "hafsa": "ncep-hafs-a",
    "hafsb": "ncep-hafs-b",
    "wpc": "wpc-ncep",
    "coamps": "coamps-tc",
    "ctcx": "coamps-ctcx",
    "nhc": "nhc",
    "era5": "era5",
}

# Available metget model types
MODEL_TYPES = {
    "gfs": "synoptic",
    "gefs": "ensemble",
    "nam": "synoptic",
    "hwrf": "synoptic-storm",
    "hrrr": "synoptic",
    "hrrr-alaska": "synoptic",
    "hafsa": "synoptic-storm",
    "hafsb": "synoptic-storm",
    "wpc": "synoptic",
    "coamps": "synoptic-storm",
    "ctcx": "ensemble-storm",
    "era5": "hindcast",
    "nhc": "track",
}

# Available metget variables
AVAILABLE_VARIABLES = {"wind_pressure", "rain", "temperature", "humidity", "ice"}

# Available metget formats
AVAILABLE_FORMATS = {
    "raw": "raw",
    "ascii": "ascii",
    "owi-ascii": "owi-ascii",
    "owi-netcdf": "owi-netcdf",
    "hec-netcdf": "hec-netcdf",
    "generic-netcdf": "hec-netcdf",
    "delft3d": "delft3d",
}


def get_metget_available_model_list() -> str:
    """
    This method is used to return a comma separated list of available models

    Returns:
        A comma separated list of available models
    """
    mlist = ""
    for m in AVAILABLE_MODELS:
        if len(mlist) == 0:
            mlist += m
        else:
            mlist += ", " + m
    return mlist



def test_cli():
    """
    Test fires the CLI. This doesn't do much except check that argparse can fire up correctly

    Returns:
        None
    """
    from metget.metget_client import metget_client_cli

    metget_client_cli()



###################################################################################################
# MIT License
#
# Copyright (c) 2023 The Water Institute
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author: Zach Cobell
# Contact: zcobell@thewaterinstitute.org
# Organization: The Water Institute
#
###################################################################################################
import argparse


def metget_credits(args: argparse.Namespace) -> None:
    """
    This method is used to get the number of credits available

    Args:
        args: The arguments passed to the command line
    """
    import json

    import prettytable
    import requests

    from .metget_environment import get_metget_environment_variables

    env = get_metget_environment_variables(args)

    url = env["endpoint"] + "/credits"
    headers = {"x-api-key": env["apikey"]}
    response = requests.get(url, headers=headers)

    if args.format == "json":
        credits_balance = response.json()["body"]
        if (
            credits_balance["credit_limit"] == 0
            and credits_balance["credit_balance"] == 0
        ):
            credits_balance["credit_balance"] = "Unlimited"
            credits_balance["credit_limit"] = "Unlimited"
        print(json.dumps(credits_balance, indent=2))
    elif args.format == "pretty":
        table = prettytable.PrettyTable(
            ["Credit Limit", "Credits Used", "Credit Balance"]
        )
        credit_limit = response.json()["body"]["credit_limit"]
        credit_balance = response.json()["body"]["credit_balance"]
        if credit_limit == 0 and credit_balance == 0:
            credit_limit = "Unlimited"
            credit_balance = "Unlimited"
        table.add_row(
            [
                credit_limit,
                response.json()["body"]["credits_used"],
                credit_balance,
            ]
        )
        # print("Credit status for apikey: " + env["apikey"])
        print(table)
    else:
        raise RuntimeError("Invalid format: " + args.format)



###################################################################################################
# MIT License
#
# Copyright (c) 2023 The Water Institute
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author: Zach Cobell
# Contact: zcobell@thewaterinstitute.org
# Organization: The Water Institute
#
###################################################################################################

import argparse
import contextlib
import json
import time
from datetime import datetime, timezone
from typing import Tuple, Union

from .metget_data import AVAILABLE_MODELS


class MetGetBuildRest:
    def __init__(
        self, metget_api_server: str, metget_api_key: str, metget_api_version: int
    ):
        """
        Constructor

        Args:
            metget_api_server (str): MetGet API server
            metget_api_key (str): MetGet API key
            metget_api_version (int): MetGet API version
        """
        self.__metget_api_server = metget_api_server
        self.__metget_api_key = metget_api_key
        self.__metget_api_version = metget_api_version

    @staticmethod
    def parse_domain_data(domain_list: list, level: int, tau: int) -> dict:
        """
        Parses the domain data from the given list and returns a dictionary

        Args:
            domain_list (list): List of domain data
            level (int): Level of the domain
            tau (int): Forecast skipping time

        Returns:
            dict: Dictionary containing the domain data
        """

        storm = None
        ensemble_member = None
        basin = None
        advisory = None
        year = None

        model = domain_list[0]
        if "hwrf" in model or "coamps" in model or "hafs" in model:
            if len(model.split("-")) < 2:
                msg = "Must include storm name with HWRF/COAMPS/HAFS request as 'hwrf-storm' or 'coamps-storm'"
                raise RuntimeError(msg)
            storm = model.split("-")[1]
            model = model.split("-")[0]
        elif "gefs" in model:
            if len(model.split("-")) < 2:
                msg = "Must include ensemble member with GEFS request as 'gefs-ensemble_member'"
                raise RuntimeError(msg)
            ensemble_member = model.split("-")[1]
            model = "gefs"
        elif "ctcx" in model:
            if len(model.split("-")) < 3:
                msg = "Must include storm name and ensemble member with CTCX request as 'ctcx-storm-ensemble_member'"
                raise RuntimeError(msg)
            ensemble_member = model.split("-")[2]
            storm = model.split("-")[1]
            model = "ctcx"
        elif "nhc" in model:
            if len(model.split("-")) < 4:
                msg = "Must include basin, storm name, and advisory with NHC request as 'nhc-basin-storm-advisory' or 'nhc-basin-storm-year-advisory'"
                raise RuntimeError(msg)
            keys = model.split("-")
            if len(keys) == 5:
                year = int(keys[1])
                basin = keys[2]
                storm = keys[3]
                advisory = keys[4]
                with contextlib.suppress(ValueError):
                    advisory = f"{int(advisory):03d}"
            else:
                year = datetime.now(timezone.utc).year
                basin = keys[1]
                storm = keys[2]
                advisory = keys[3]
                with contextlib.suppress(ValueError):
                    advisory = f"{int(advisory):03d}"
            model = "nhc"
        else:
            if len(model.split("-")) > 1:
                raise RuntimeError(
                    "Model '"
                    + model
                    + "' does not support additional '-' separated arguments"
                )

        res = float(domain_list[1])
        x0 = float(domain_list[2])
        y0 = float(domain_list[3])
        x1 = float(domain_list[4])
        y1 = float(domain_list[5])

        if model not in AVAILABLE_MODELS:
            raise RuntimeError("Specified model '" + model + "' is not available")

        xmax = max(x0, x1)
        xmin = min(x0, x1)
        ymax = max(y0, y1)
        ymin = min(y0, y1)
        res = abs(res)
        if res <= 0:
            msg = "Specified model resolution is invalid"
            raise RuntimeError(msg)

        if model == "hwrf" or model == "hafsa" or model == "hafsb" or model == "coamps":
            return {
                "name": AVAILABLE_MODELS[model] + "-" + storm,
                "service": AVAILABLE_MODELS[model],
                "storm": storm,
                "x_init": xmin,
                "y_init": ymin,
                "x_end": xmax,
                "y_end": ymax,
                "di": res,
                "dj": res,
                "tau": tau,
                "level": level,
            }
        elif model == "ctcx":
            return {
                "name": AVAILABLE_MODELS[model] + "-" + storm + "-" + ensemble_member,
                "service": AVAILABLE_MODELS[model],
                "storm": storm,
                "tau": tau,
                "ensemble_member": ensemble_member,
                "x_init": xmin,
                "y_init": ymin,
                "x_end": xmax,
                "y_end": ymax,
                "di": res,
                "dj": res,
                "level": level,
            }
        elif model == "gefs":
            return {
                "name": model,
                "service": AVAILABLE_MODELS[model],
                "ensemble_member": ensemble_member,
                "x_init": xmin,
                "y_init": ymin,
                "x_end": xmax,
                "y_end": ymax,
                "di": res,
                "dj": res,
                "level": level,
            }
        elif model == "nhc":
            return {
                "name": model,
                "service": AVAILABLE_MODELS[model],
                "basin": basin,
                "storm": storm,
                "storm_year": year,
                "advisory": advisory,
                "x_init": xmin,
                "y_init": ymin,
                "x_end": xmax,
                "y_end": ymax,
                "di": res,
                "dj": res,
                "level": level,
            }
        else:
            return {
                "name": model,
                "service": AVAILABLE_MODELS[model],
                "x_init": xmin,
                "y_init": ymin,
                "x_end": xmax,
                "y_end": ymax,
                "di": res,
                "dj": res,
                "level": level,
            }

    @staticmethod
    def parse_command_line_domains(domain_list: list, initialization_skip: int):
        """
        Generates the domains from the given list of domains

        Args:
            domain_list (list): List of domains
            initialization_skip (int): Initialization skip time

        Returns:
            list: List of domains as dictionaries
        """
        idx = 0
        domains = []
        for d in domain_list:
            j = MetGetBuildRest.parse_domain_data(d, idx, initialization_skip)
            domains.append(j)
            idx += 1
        return domains

    @staticmethod
    def generate_request_json(**kwargs) -> dict:
        """
        Generates the request JSON for the given arguments

        Returns:
            dict: Request JSON
        """
        import getpass
        import socket

        request_data = {
            "version": "0.0.1",
            "creator": kwargs.get(
                "username", getpass.getuser() + "." + socket.gethostname()
            ),
            "background_pressure": kwargs.get("background_pressure", 1013.0),
            "backfill": kwargs.get("backfill", False),
            "nowcast": kwargs.get("nowcast", False),
            "multiple_forecasts": kwargs.get("multiple_forecasts", False),
            "start_date": str(kwargs.get("start_date")),
            "end_date": str(kwargs.get("end_date")),
            "format": kwargs.get("format", "owi-ascii"),
            "data_type": kwargs.get("data_type", "wind_pressure"),
            "time_step": kwargs.get("time_step", 3600),
            "domains": kwargs.get("domains"),
            "compression": kwargs.get("compression", False),
            "epsg": kwargs.get("epsg", 4326),
            "filename": kwargs.get("filename", "metget_data"),
        }

        if kwargs.get("strict", False):
            request_data["strict"] = True
        else:
            request_data["strict"] = False

        if kwargs.get("dry_run", False):
            request_data["dry_run"] = True
        else:
            request_data["dry_run"] = False

        if kwargs.get("save_json_request", False):
            with open("request.json", "w") as f:
                f.write(json.dumps(request_data, indent=2))

        return request_data

    def make_metget_request(self, request_json: dict) -> Tuple[str, int]:
        """
        Makes a request to the MetGet API and returns the data id and status code

        Args:
            request_json (dict): Request JSON

        Returns:
            Tuple[str, int]: Data id and status code
        """
        import requests

        headers = {"x-api-key": self.__metget_api_key}
        r = requests.post(
            self.__metget_api_server + "/build", headers=headers, json=request_json
        )
        if r.status_code != 200:
            if r.text:
                with open("metget.debug", "a") as f:
                    f.write(r.text)
            raise RuntimeError(
                "Request to MetGet was returned status code = " + str(r.status_code)
            )
        return_data = json.loads(r.text)
        data_id = return_data["body"]["request_id"]
        status_code = return_data["statusCode"]
        if status_code != 200:
            with open("metget.debug", "a") as f:
                f.write(
                    "[WARNING]: MetGet returned status code " + str(status_code) + "\n"
                )
                f.write(str(return_data["body"]["error_text"]))
        return data_id, status_code

    def download_metget_data(
        self,
        data_id: str,
        sleep_time: int,
        max_wait: int,
        output_directory: Union[str, None],
    ) -> None:
        """
        Downloads the data from the MetGet API

        Args:
            data_id (str): Data id
            sleep_time (int): Time to sleep between status checks
            max_wait (int): Maximum time to wait for data to appear
            output_directory (Union[str, None]): Output directory

        Returns:
            None
        """
        import os
        from datetime import datetime, timedelta, timezone

        import requests

        from .spinnerlogger import SpinnerLogger

        # ...Wait time
        end_time = datetime.now(timezone.utc) + timedelta(hours=max_wait)

        # ...Timing information
        request_start_time = datetime.now(timezone.utc)

        # ...Wait for request data to appear
        tries = 0
        data_ready = False
        status = None

        spinner = SpinnerLogger()
        print(f"Waiting for request id {data_id:s}", flush=True)
        spinner.start()

        return_data = None
        data_url = None

        while datetime.now(timezone.utc) <= end_time:
            tries += 1
            try:
                data_url, status = self.check_metget_status(data_id)
                spinner.set_text(SpinnerLogger.standard_log(tries, status))
                if status == "completed":
                    spinner.succeed()
                    # ...Parse the return to get data
                    data_ready = True
                    flist_url = data_url + "/filelist.json"
                    u = requests.get(flist_url)
                    if u.status_code == 200:
                        return_data = json.loads(u.text)
                        with open("filelist.json", "w") as jsonfile:
                            jsonfile.write(
                                json.dumps(return_data, indent=2, sort_keys=True)
                            )
                        break
                elif status == "error":
                    spinner.fail("Request could not be completed")
                    return
                else:
                    time.sleep(sleep_time)
                    continue
            except KeyboardInterrupt:
                spinner.fail("Process was ended by the user")
                raise

        # ...Download files
        if data_ready:
            file_list = return_data["output_files"]
            for f in file_list:
                if output_directory is not None:
                    if not os.path.exists(output_directory):
                        msg = f"Output directory does not exist: {output_directory:s}"
                        raise RuntimeError(msg)
                    ff = os.path.join(output_directory, f)
                else:
                    ff = f

                time_stamp = datetime.now(timezone.utc).strftime(
                    "%Y-%m-%d %H:%M:%S UTC"
                )
                spinner.start(f"[{time_stamp:s}]: Getting file: {f:s}")
                with requests.get(data_url + "/" + f, stream=True) as r:
                    r.raise_for_status()
                    with open(ff, "wb") as wind_file:
                        for chunk in r.iter_content(chunk_size=8192):
                            wind_file.write(chunk)
                spinner.succeed()

            request_end_time = datetime.now(timezone.utc)
            request_duration = request_end_time - request_start_time
            hours, remainder = divmod(request_duration.total_seconds(), 3600)
            minutes, seconds = divmod(remainder, 60)

            time_stamp = datetime.now(timezone.utc).strftime("%Y-%m-%d %H:%M:%S UTC")
            spinner.succeed(
                "[{:s}]: Elapsed time: {:d}h{:02d}m{:02d}s".format(
                    time_stamp, int(hours), int(minutes), int(seconds)
                )
            )
        else:
            if status == "restore":
                print(
                    "[WARNING]: Data for request "
                    + data_id
                    + " did not become ready before the max-wait time expired. "
                    "You can rerun and ask for this request by id"
                )
            elif status == "running":
                print(
                    "[WARNING]: Data for request "
                    + data_id
                    + " is still being constructed when the max-wait time expired. Please check on it later"
                )
            elif status == "queued":
                print(
                    "[WARNING]: Data for request "
                    + data_id
                    + " is still queued. If this does not change soon, please contact an administrator"
                )
            else:
                print("[ERROR]: Data has not become available due to an unknown error")
            return

    def check_metget_status(
        self,
        data_id: str,
    ) -> Tuple[str, str]:
        """
        Checks the status of a MetGet request

        Args:
            data_id (str): Data id

        Returns:
            Tuple[str, str]: Data url and status
        """
        if self.__metget_api_version == 2:
            response = self.__check_metget_status_v2(data_id)
        else:
            msg = f"Invalid MetGet API version. Must be 2. Got {self.__metget_api_version:d}"
            raise RuntimeError(msg)

        json_response = json.loads(response)
        status = json_response["body"]["status"]
        data_url = json_response["body"]["destination"]
        return data_url, status

    def __check_metget_status_v2(self, data_id: str) -> str:
        """
        Checks the status of a MetGet request using the v2 API

        Args:
            data_id (str): Data id

        Returns:
            str: response text
        """
        import requests

        headers = {"x-api-key": self.__metget_api_key}
        request_params = {"request-id": data_id}
        response = requests.get(
            self.__metget_api_server + "/check", headers=headers, params=request_params
        )
        return response.text


def metget_build(args: argparse.Namespace) -> None:
    """
    This method is used to build a MetGet request

    Args:
        args: The arguments passed to the command line

    Returns:
        None
    """
    from .metget_data import AVAILABLE_FORMATS, AVAILABLE_VARIABLES
    from .metget_environment import get_metget_environment_variables

    # ...Get the environment variables
    environment = get_metget_environment_variables(args)

    # ...Initialize the metget client
    client = MetGetBuildRest(
        environment["endpoint"], environment["apikey"], environment["api_version"]
    )

    # ...Check for required arguments
    if not args.request:
        if not args.start:
            print("[ERROR]: Must provide '--start'")
            exit(1)
        if not args.end:
            print("[ERROR]: Must provide '--end'")
            exit(1)
        if not args.timestep:
            print("[ERROR]: Must provice '--timestep'")
            exit(1)
        if not args.output:
            print("[ERROR]: Must provide '--output'")
            exit(1)

        output_format = AVAILABLE_FORMATS[args.format]
        if (output_format == "delft3d" or output_format == "hec-netcdf") and len(
            args.domain
        ) > 1:
            print("[ERROR]: " + args.format + " does not support more than one domain.")
            exit(1)

        if args.variable not in AVAILABLE_VARIABLES:
            print(f"ERROR: Invalid variable '{args.variable:s}' selected")
            exit(1)

        if args.format not in AVAILABLE_FORMATS:
            print(f"ERROR: Invalid output format '{args.format:s}' selected")
            exit(1)

        # ...Building the request
        request_data = MetGetBuildRest.generate_request_json(
            analysis=args.analysis,
            multiple_forecasts=args.multiple_forecasts,
            start_date=args.start,
            end_date=args.end,
            format=output_format,
            data_type=args.variable,
            backfill=args.backfill,
            time_step=args.timestep,
            domains=MetGetBuildRest.parse_command_line_domains(
                args.domain, args.initialization_skip
            ),
            compression=args.compression,
            epsg=args.epsg,
            filename=args.output,
            strict=args.strict,
            dry_run=args.dryrun,
            save_json_request=args.save_json_request,
        )

        data_id, status_code = client.make_metget_request(request_data)
        if not args.dryrun and status_code == 200:
            client.download_metget_data(
                data_id,
                args.check_interval,
                args.max_wait,
                args.output_directory,
            )
        else:
            print(status_code)

    else:
        client.download_metget_data(
            args.request,
            args.check_interval,
            args.max_wait,
            args.output_directory,
        )



import argparse
from urllib.parse import urlencode

import requests_mock

from metget.metget_track import MetGetTrack

from .track_json import NHC_IAN_BESTRACK_JSON, NHC_IAN_FORECAST_JSON

METGET_DMY_ENDPOINT = "https://metget.server.dmy"
METGET_DMY_APIKEY = "1234567890"
METGET_API_VERSION = 2


def test_best_track(capfd) -> None:
    """
    Tests the best track data
    Args:
        capfd: pytest fixture to capture stdout and stderr

    Returns:
        None
    """
    import json
    import os

    args = argparse.Namespace()
    args.type = "besttrack"
    args.storm = 9
    args.year = 2022
    args.basin = "al"

    # ...Test using the environment variables
    os.environ["METGET_ENDPOINT"] = METGET_DMY_ENDPOINT
    os.environ["METGET_API_KEY"] = METGET_DMY_APIKEY
    os.environ["METGET_API_VERSION"] = str(METGET_API_VERSION)
    args.endpoint = None
    args.apikey = None
    args.api_version = None

    with requests_mock.Mocker() as m:
        url = (
            METGET_DMY_ENDPOINT
            + "/stormtrack?"
            + urlencode(
                {
                    "type": "best",
                    "storm": "09",
                    "basin": "al",
                    "year": "2022",
                }
            )
        )
        m.get(url, json=NHC_IAN_BESTRACK_JSON)
        s = MetGetTrack(args)
        s.get_track()
        out, err = capfd.readouterr()
        out_dict = json.loads(out)
        assert out_dict == NHC_IAN_BESTRACK_JSON["body"]["geojson"]


def test_forecast_track(capfd) -> None:
    """
    Tests the forecast track data
    Args:
        capfd: pytest fixture to capture stdout and stderr

    Returns:
        None
    """
    import json

    args = argparse.Namespace()
    args.type = "forecast"
    args.storm = 9
    args.year = 2022
    args.basin = "al"
    args.advisory = 12
    args.endpoint = METGET_DMY_ENDPOINT
    args.apikey = METGET_DMY_APIKEY
    args.api_version = METGET_API_VERSION

    with requests_mock.Mocker() as m:
        url = (
            METGET_DMY_ENDPOINT
            + "/stormtrack?"
            + urlencode(
                {
                    "type": "forecast",
                    "storm": "09",
                    "basin": "al",
                    "advisory": "012",
                    "year": "2022",
                }
            )
        )
        m.get(url, json=NHC_IAN_FORECAST_JSON)
        s = MetGetTrack(args)
        s.get_track()
        out, err = capfd.readouterr()
        out_dict = json.loads(out)
        assert out_dict == NHC_IAN_FORECAST_JSON["body"]["geojson"]



###################################################################################################
# MIT License
#
# Copyright (c) 2023 The Water Institute
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author: Zach Cobell
# Contact: zcobell@thewaterinstitute.org
# Organization: The Water Institute
#
###################################################################################################
import argparse


def metget_version() -> str:
    """
    This method is used to get the version of the MetGet client

    Returns:
        The version of the MetGet client
    """
    from . import __version__ as version

    return version


def get_metget_environment_variables(args: argparse.Namespace) -> dict:
    """
    This method is used to get the environment variables for the endpoint and
    apikey

    Args:
        args: The arguments passed to the command line

    Returns:
        A dictionary containing the endpoint and apikey
    """
    import os

    if not args.endpoint:
        if "METGET_ENDPOINT" not in os.environ:
            msg = "No endpoint found."
            raise RuntimeError(msg)
        else:
            endpoint = os.environ["METGET_ENDPOINT"]
    else:
        endpoint = args.endpoint

    if not args.apikey:
        if "METGET_API_KEY" not in os.environ:
            msg = "No API key was found."
            raise RuntimeError(msg)
        else:
            apikey = os.environ["METGET_API_KEY"]
    else:
        apikey = args.apikey

    if not args.api_version:
        if "METGET_API_VERSION" not in os.environ:
            api_version = 2
        else:
            api_version = int(os.environ["METGET_API_VERSION"])
    else:
        api_version = args.api_version

    return {"endpoint": endpoint, "apikey": apikey, "api_version": api_version}



###################################################################################################
# MIT License
#
# Copyright (c) 2023 The Water Institute
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author: Zach Cobell
# Contact: zcobell@thewaterinstitute.org
# Organization: The Water Institute
#
###################################################################################################


def initialize_build_cli(subparsers):
    """
    This method is used to initialize the build subparser

    Args:
        subparsers: The build subparser

    Returns:
        None
    """
    from datetime import datetime

    from .metget_build import metget_build
    from .metget_data import get_metget_available_model_list

    build = subparsers.add_parser(
        name="build",
        help="Generate a MetGet build request",
    )
    build.set_defaults(func=metget_build)

    mlist = get_metget_available_model_list()
    build.add_argument(
        "--domain",
        help="Wind domain specification. Model may be any of ["
        + mlist
        + "]. Resolution and corners are decimal degrees"
        " For HWRF/COAMPS, the model can be listed as 'hwrf-[stormname]' or 'coamps-[stormname]'."
        " For GEFS, the ensemble member can be specified as 'gefs-[ensemble_member]'. For NHC data"
        " specify as 'nhc-basin-storm_number-advisory_number' where basin is a two letter string denoting"
        " the basin (al, ep, wp), storm number is the id of the storm (not the name), and the advisory number"
        " is the advisory to use to build the merged data (or 0 for best-track data only).",
        nargs=6,
        metavar=("model", "resolution", "x0", "y0", "x1", "y1"),
        action="append",
    )
    build.add_argument(
        "--start",
        help="Start time",
        type=datetime.fromisoformat,
        metavar="YYYY-MM-DD hh:mm",
    )
    build.add_argument(
        "--end",
        help="End time",
        type=datetime.fromisoformat,
        metavar="YYYY-MM-DD hh:mm",
    )
    build.add_argument(
        "--timestep", help="Time step of winds in seconds", metavar="dt", type=int
    )
    build.add_argument(
        "--analysis", help="Generate analysis wind fields", action="store_true"
    )
    build.add_argument(
        "--multiple-forecasts",
        help="Allow the use of multiple forecast wind fields",
        action="store_true",
    )
    build.add_argument(
        "--initialization-skip",
        help="For COAMPS-TC, skip some initialization period of each forecast. "
        "This is considered a discard period where forecast times < [skip] hours are "
        "discarded when selecting the data",
        type=int,
        metavar="h",
        required=False,
        default=0,
    )
    build.add_argument(
        "--output", help="Base name of output data", type=str, metavar="s"
    )
    build.add_argument(
        "--format",
        help="Output format (raw, owi-ascii, owi-netcdf, generic-netcdf (hec-netcdf), delft3d)",
        metavar="f",
        default="owi-ascii",
    )
    build.add_argument(
        "--variable",
        help="Variable to request from MetGet (wind_pressure, rain, humidity, temperature, ice)",
        metavar="v",
        default="wind_pressure",
    )
    build.add_argument(
        "--check-interval",
        help="Time between status checks (default=10s)",
        metavar="t",
        default=10,
        type=float,
    )
    build.add_argument(
        "--max-wait",
        help="Maximum wait time for the request to complete in hours (default=24)",
        metavar="h",
        default=24,
        type=float,
    )
    build.add_argument(
        "--strict",
        action="store_true",
        help="Do not allow MetGet to make due with what it has. Force the response to match the request",
    )
    build.add_argument(
        "--backfill",
        action="store_true",
        help="Backfill data from lower priority domains",
        default=False,
    )
    build.add_argument(
        "--epsg",
        help="Coordinate system of the specified domain and output data (default: 4326)",
        required=False,
        metavar="#",
        type=int,
        default=4326,
    )
    build.add_argument("--dryrun", help="Perform dry run only", action="store_true")
    build.add_argument(
        "--request",
        help="Check on and download specified request id",
        type=str,
        metavar="request_id",
    )
    build.add_argument(
        "--compression",
        action="store_true",
        help="For ASCII based data formats, use gzip compression in the retreived files",
        default=False,
    )
    build.add_argument(
        "--save-json-request",
        action="store_true",
        help="Save the json request sent to the MetGet API as request.json",
        default=False,
    )
    build.add_argument(
        "--output-directory",
        type=str,
        metavar="s",
        default=None,
        help="Directory to save output files to",
    )


def initialize_credits_cli(subparsers):
    """
    This method is used to initialize the credits subparser

    Args:
        subparsers: The credits subparser

    Returns:
        None
    """
    from .metget_credits import metget_credits

    api_credits = subparsers.add_parser(
        "credits", help="Check the number of credits available"
    )
    api_credits.add_argument(
        "--format",
        help="Output format (json, pretty)",
        default="pretty",
        type=str,
    )
    api_credits.set_defaults(func=metget_credits)


def initialize_status_cli(subparsers):
    from datetime import datetime

    from .metget_data import get_metget_available_model_list
    from .metget_status import metget_status

    status = subparsers.add_parser(
        "status", help="Check the status of the available data"
    )
    status.set_defaults(func=metget_status)
    mlist = get_metget_available_model_list()
    status.add_argument(
        "model", help="Name of model to get status for (" + mlist + ")", type=str
    )
    status.add_argument(
        "--start",
        help="Earlier date to retrieve status for",
        type=datetime.fromisoformat,
        metavar="YYYY-MM-DD hh:mm",
    )
    status.add_argument(
        "--end",
        help="Later date to retrieve status for",
        type=datetime.fromisoformat,
        metavar="YYYY-MM-DD hh:mm",
    )
    status.add_argument(
        "--basin",
        help="For NHC based data, request a specific basin",
        type=str,
        metavar="s",
    )
    status.add_argument(
        "--year",
        help="For storm based data (nhc, coamps, hwrf), the year in which the storm occured",
        type=int,
        metavar="YYYY",
    )
    status.add_argument(
        "--storm",
        help="For storm based data, request a specific storm",
        type=str,
        metavar="s",
    )
    status.add_argument(
        "--ensemble-member",
        help="Ensemble member to request data for",
        type=str,
        metavar="s",
    )
    status.add_argument(
        "--format", help="Output format (json, pretty)", metavar="f", default="pretty"
    )
    status.add_argument(
        "--complete", help="Only show data which is complete", action="store_true"
    )


def initialize_track_cli(subparsers) -> None:
    """
    This method is used to initialize the track subparser

    Returns:
        None
    """
    from .metget_track import metget_track

    track = subparsers.add_parser(
        "track", help="Get the storm track data for a NHC storm"
    )
    track.set_defaults(func=metget_track)
    track.add_argument(
        "--year", help="NHC Storm year to get track data for", type=int, metavar="n"
    )
    track.add_argument(
        "--storm", help="NHC Storm number to get track data for", type=int, metavar="n"
    )
    track.add_argument(
        "--basin",
        help="NHC Storm basin to get track data for (al, ep, wp)",
        type=str,
        metavar="s",
        default="al",
    )
    track.add_argument(
        "--advisory", help="NHC Storm advisory to get", type=str, metavar="s"
    )
    track.add_argument(
        "--type",
        help="Type of track data to get (besttrack or forecast)",
        type=str,
        metavar="s",
    )


def metget_client_cli() -> None:
    """
    Main function for command line interface
    """
    import argparse

    from .metget_environment import metget_version

    p = argparse.ArgumentParser(
        description="Client for interaction with a MetGet API instance",
        prog="metget",
    )
    p.add_argument(
        "--version", action="version", version="%(prog)s " + metget_version()
    )
    p.add_argument(
        "--apikey", help="API key for access to MetGet", type=str, metavar="s"
    )
    p.add_argument("--endpoint", help="MetGet API endpoint", type=str, metavar="s")
    p.add_argument(
        "--api-version",
        type=int,
        help="MetGet API version. Default is 1. When using the k8s MetGet API, this should be 2.",
        metavar="n",
    )

    subparsers = p.add_subparsers(help="Sub-command help")
    initialize_build_cli(subparsers)
    initialize_status_cli(subparsers)
    initialize_track_cli(subparsers)
    initialize_credits_cli(subparsers)

    args = p.parse_args()
    if "func" in args:
        args.func(args)
    else:
        p.print_help()


if __name__ == "__main__":
    metget_client_cli()
